######################################################################
#
# Run this code using: Read("CocycleFinding.g") from the containing folder.
#
# Construction of the umbral groups.  Most of the groups were constructed by Duncan and the rest by Whalen.
# Cocycles were computed by Whalen.
# This code should take ~10 mintes to run the first time, and milliseconds on subsequent runs, because we only
# calculate the resolutions if the symbol we are going to assign to them is undefined.  If you skip evaluating rr4,
# the first runtime drops to about 20s.
#
# Most of the umbral groups have the following information
# !.MoonshineR is a resolution of the group up to order 4
# !.MoonshineClassReps is a list of representatives of all the conjugacy classes
# !.MoonshineClassNames is the names of the corresponding classes in GAP's character table.  This may not correspond to UMatNL
# !.Moonshinens are the ns of the corresponding classes in the n|h of UMatNL
# !.Moonshinehs are the hs
# !.Moonshinev are the vs from which the multiplier system is constructed, using the original Umbral Moonshine paper.  Some
#     groups have nonstandard multipliers.  See the comments for the vs.
# !.MoonshineVec is the cocycle corresponding to the multiplier system of the McKay-Thompson series
# !.MoonshineVecPrime is the other choice of cocycle in the case that it is ambiguous from the information I know
# !.MoonshineExp is the exponent of the resolution
#
# CURRENT STATE
# gg2: has classes, ns, and hs, but no cocycle information
# gg3: complete, but only in the Sylow subgroups.  Cocycle for Sylow 2 is ambiguous.
# gg4: complete
# gg5: complete
# gg6: complete, but ambiguous cocycle
# gg63: complete, but note, n|h=2|101 is written in the construction for 2B which has n|h=2|1
# gg7: complete, but ambiguous cocycle
# gg8: complete
# gg9: complete
# gg10: complete
# gg105: complete.
# gg12: complete
# gg124: complete
# gg13: complete
# gg147: complete
# gg16: complete
# gg18: complete
# gg189: complete
# gg2211: complete
# gg25: complete
# gg3015: trivial
# gg3061015: complete
# gg4623: trivial
#
# Note that for l=0,1 mod 4, v=3 mod 4, l=2,3 mod 4, v=1 mod 4
# Note that for l=0 mod 3, v=2 mod 3, l=2 mod 4, v=1 mod 3
#
######################################################################

LoadPackage("HAP");

#------------------------------------------------#
# Construction of G^{(2)}
#------------------------------------------------#

gg2:=MathieuGroup(24);
gg2!.Moonshinem := 2;


gg2!.MoonshineClassNames := [ "1a", "5a", "3a", "2a", "4a", "2b", "4b", "7a", "7b", "14a", "14b", "23a", "23b", "15a", "15b", "3b", "6a",
"12a", "11a", "10a", "21a", "21b", "4c", "8a", "6b", "12b" ];
gg2!.MoonshineClassNames :=[ (), (1,21,15,24,2)(3,18,22,16,19)(4,17,6,9,20)(5,11,10,12,23), (1,10,22)(2,11,18)(3,24,5)(7,14,13)(12,16,21)(15,
  23,19), (1,7)(2,21)(4,8)(10,14)(11,12)(13,22)(16,18)(17,20), (1,12,7,11)(2,22,21,13)(3,19)(4,17,8,20)(5,23)(6,
    9)(10,16,14,18)(15,24), (1,8)(2,18)(3,6)(4,7)(5,24)(9,19)(10,22)(11,17)(12,20)(13,14)(15,23)(16,21),
    (1,22,8,10)(2,17,18,11)(3,23,6,15)(4,14,7,13)(5,9,24,19)(12,21,20,16), (1,2,18,14,4,13,17)(3,5,23,9,24,19,6)(7,
      21,16,10,8,22,20), (1,17,13,4,14,18,2)(3,6,19,24,9,23,5)(7,20,22,8,10,16,21), (1,8,2,22,18,20,14,7,4,21,13,16,
        17,10)(3,24,5,19,23,6,9)(11,12), (1,22,14,21,17,8,18,7,13,10,2,20,4,16)(3,19,9,5,6,24,23)(11,12),
        (1,8,12,19,20,17,14,24,21,7,6,5,15,10,18,23,13,9,11,4,22,3,2), (1,2,3,22,4,11,9,13,23,18,10,15,5,6,7,21,24,14,17,
          20,19,12,8), (1,4,17,11,24)(2,19,22,12,21,6,23,15,7,10,13,3,14,8,9)(5,18,16), (1,4,17,11,24)(2,3,15,12,9,13,23,
            22,8,10,6,19,14,7,21)(5,16,18), (1,21,14)(2,23,5)(3,8,4)(6,16,20)(7,17,22)(9,19,11)(10,13,24)(12,15,18),
            (1,19,21,11,14,9)(2,12,23,15,5,18)(3,13,8,24,4,10)(6,22,16,7,20,17), (1,5,19,18,21,2,11,12,14,23,9,15)(3,22,13,
              16,8,7,24,20,4,17,10,6), (1,2,15,13,5,9,23,4,7,22,11)(3,21,19,16,12,24,18,6,17,20,8),
              (1,19,3,9,2,7,18,12,23,22)(4,6,16,11,10,17,5,15,24,14)(8,21)(13,20), (1,4,18,12,5,15,10,7,13,22,3,19,24,23,8,2,
                17,20,6,11,16)(9,21,14), (1,3,16,22,11,13,6,7,20,10,17,15,2,5,8,12,23,18,24,4,19)(9,21,14),
                (1,11)(2,18,14,6)(3,17,20,22)(4,24,10,13)(7,9,19,23)(8,16), (1,16,11,8)(2,24,18,10,14,13,6,4)(3,9,17,19,20,23,22,
                  7)(12,15), (1,24)(2,22,23)(3,6,8,5,12,19)(7,20,18,11,15,21)(9,17,10)(13,16), (1,16,24,13)(2,10,22,9,23,17)(3,7,
                    6,20,8,18,5,11,12,15,19,21)(4,14) ];
gg2!.Moonshinens := [1,5,3,2,4,2,4,7,7,14,14,23,23,15,15,3,6,12,11,10,21,21,4,8,6,12];
gg2!.Moonshinehs := [1,1,1,1,2,2,4,1,1, 1, 1, 1, 1, 1, 1,3,6, 2, 1, 2, 3, 3,1,1,1,12];
gg2!.Moonshinev:=1;

#------------------------------------------------#
# Construction of G^{(3)}
#------------------------------------------------#

# 2.M12
gg3:=Group([ (2,3,4,5,6,7,8,9,10,11,12)(14,15,16,17,18,19,20,21,22,23,24), (1,8)(4,24)(5,7)(9,22)(10,21)(12,16)(13,20)(17,19) ]);
gg3!.Moonshinem:=3;

gg3s2 := Group( [ ( 1,13)( 2,14)( 3,15)( 4,16)( 5,17)( 6,18)( 7,19)( 8,20)( 9,21)(10,22)(11,23)(12,24),
  ( 3,10)( 4, 7)( 8, 9)(11,12)(15,22)(16,19)(20,21)(23,24), ( 4,12)( 5, 6)( 7,11)( 8, 9)(16,24)(17,18)
    (19,23)(20,21), ( 1,13)( 2,14)( 4,19,12,23)( 5, 8, 6, 9)( 7,24,11,16)(17,20,18,21),
  ( 1, 2)( 3,10)( 4,16)( 5, 6)( 7,19)( 8, 9)(11,23)(12,24)(13,14)(15,22)(17,18)(20,21),
  ( 1, 3,13,15)( 2,10,14,22)( 4,19,16, 7)( 5, 8,17,20)( 6, 9,18,21)(11,12,23,24),
  ( 1,17,13, 5)( 2,18,14, 6)( 3, 9,15,21)( 4,24,16,12)( 7,23,19,11)( 8,22,20,10) ] ); #SylowSubgroup(gg3,2);
gg3s3 := Group([ (1,9,10)(2,12,18)(3,16,23)(4,11,15)(6,14,24)(13,21,22), (1,2,15)(3,13,14)(4,9,12)(5,8,19)
(6,23,22)(7,17,20)(10,18,11)(16,21,24), (2,12,18)(3,23,16)(4,15,11)(5,8,19)(6,14,24)(7,17,20) ]); #SylowSubgroup(gg3,3);
gg3s2!.Moonshinem := 3;
gg3s3!.Moonshinem := 3;

N1:=gg3s2;
N2:= Group([ (1,2)(3,10)(4,24)(7,23)(11,19)(12,16)(13,14)(15,22), (4,12)(5,6)(7,11)(8,9)(16,24)(17,18)(19,23)(20,21) ]);
N3:=Group(());
if not IsBound(rr3s2) then rr3s2 := ResolutionNormalSeries([N1,N2,N3],4); fi;
if not IsBound(rr3s3) then rr3s3 := ResolutionFiniteGroup(gg3s3,4); fi;
gg3s2!.MoonshineR := rr3s2;
gg3s3!.MoonshineR := rr3s3;

gg3!.MoonshineClassReps:=[ (), (1,13)(2,14)(3,15)(4,16)(5,17)(6,18)(7,19)(8,20)(9,21)(10,22)(11,23)(12,24), (1,8)(3,6)(4,22)(9,24)(10,16)(12,21)(13,20)(15,18), (1,20)(2,14)(3,18)(4,10)(5,17)(6,15)(7,19)(8,13)(9,12)(11,23)(16,22)(21,
    24), (1,24,22)(2,7,5)(4,8,9)(10,13,12)(14,19,17)(16,20,21), (1,12,22,13,24,10)(2,19,5,14,7,17)(3,15)(4,20,9,16,8,21)(6,18)(11,23), (1,4,24,8,22,9)(2,5,7)(3,6)(10,21,13,16,12,20)(14,17,19)(15,18),
  (1,16,24,20,22,21)(2,17,7,14,5,19)(3,18)(4,12,8,10,9,13)(6,15)(11,23), (1,2,22,9)(4,12,7,6)(5,17)(10,21,13,14)(11,23)(16,24,19,18), (1,24,2,19,22,18,9,16)(4,13,12,14,7,10,6,21)(5,11,17,23)(8,20),
  (1,12,2,7,22,6,9,4)(3,15)(5,23,17,11)(10,18,21,16,13,24,14,19), (1,24,17,4)(2,21)(3,8,18,19)(5,16,13,12)(6,7,15,20)(9,14)(10,11)(22,23), (1,3,4,19,17,18,24,8)(2,10,21,11)(5,6,12,20,13,15,16,7)(9,23,14,22),
  (1,15,4,7,17,6,24,20)(2,22,21,23)(3,16,19,5,18,12,8,13)(9,11,14,10), (1,4,22,11,24,9,19,8,18,5,14,13,16,10,23,12,21,7,20,6,17,2)(3,15), (1,16,22,23,24,21,19,20,18,17,14)(2,13,4,10,11,12,9,7,8,6,5),
  (1,2,17,6,20,7,21,12,23,10,16,13,14,5,18,8,19,9,24,11,22,4)(3,15), (1,14,17,18,20,19,21,24,23,22,16)(2,5,6,8,7,9,12,11,10,4,13), (1,24,13,12)(2,4,14,16)(3,23,15,11)(5,6,17,18)(7,22,19,10)(8,21,20,9),
  (1,9,17,2,23,13,21,5,14,11)(3,24,8,18,4,15,12,20,6,16)(7,19)(10,22), (1,21,17,14,23)(2,11,13,9,5)(3,12,8,6,4)(15,24,20,18,16), (1,4,9,15,17,12,2,20,23,6,13,16,21,3,5,24,14,8,11,18)(7,10,19,22),
  (1,16,9,3,17,24,2,8,23,18,13,4,21,15,5,12,14,20,11,6)(7,22,19,10), (1,2,15)(3,13,14)(4,12,8)(5,19,22)(6,9,23)(7,10,17)(11,18,21)(16,24,20), (1,14,15,13,2,3)(4,24,8,16,12,20)(5,7,22,17,
    19,10)(6,21,23,18,9,11), (1,7,14,22,15,17,13,19,2,10,3,5)(4,9,24,11,8,6,16,21,12,23,20,18) ];
gg3!.MoonshineClassNames:=[ "1a", "2a", "2b", "2c", "3a", "6a", "6b", "6c", "4a", "8a", "8b", "4b", "8c", "8d", "22a", "11a", "22b", "11b", "4c", "10a", "5a", "20a", "20b", "3b", "6d", "12a" ];
#gg3!.Moonshinens := [ 1, 1, 2, 2, 3, 3, 6, 6, 4, 8, 8, 4, 8, 8, 11, 11, 11, 11, 2, 5, 5, 10, 10, 3, 6, 6];
#gg3!.Moonshinehs := [ 1, 4, 1, 2, 1, 4, 1, 2, 1, 1, 1, 2, 4, 4, 4, 1, 4, 1, 8, 4,     1,  8,  8, 3, 2, 24];
gg3!.Moonshinens := [ 1, 1, 2, 2, 3, 3, 6, 6, 4, 8, 8, 4, 8, 8, 11, 11, 11, 11, 2, 5, 5, 10, 10, 3,  3,  6];
gg3!.Moonshinehs := [ 1, 4, 1, 2, 1, 4, 1, 2, 1, 1, 1, 2, 4, 4,  4,  1,  4,  1, 8, 4, 1,  8,  8, 3, 12, 24];
gg3!.Moonshinev:=5; # mod 12
gg3s2!.Moonshinev:=5;
gg3s3!.Moonshinev:=5;

gg3s2!.MoonshineClassNames:=[ "1a", "2a", "2b", "4a", "2c", "2d", "2e", "4b", "4c", "4d", "4e", "8a", "8b", "4f", "4g", "4h", "4i", "8c", "8d", "2f", "2g", "2h", "2i" ];
gg3s2!.MoonshineClassReps:=[ (), (4,12)(5,6)(7,11)(8,9)(16,24)(17,18)(19,23)(20,21), (3,10)(4,7)(8,9)(11,12)(15,22)(16,19)(20,21)(23,24), (3,15)(4,7,12,11)(5,20,6,21)(8,18,9,17)(10,
    22)(16,19,24,23), (3,22)(5,21)(6,20)(7,11)(8,18)(9,17)(10,15)(19,23), (1,2)(3,10)(4,16)(5,6)(7,19)(8,9)(11,23)(12,24)(13,14)(15,22)(17,18)(20,21),
  (1,2)(3,15)(4,16)(5,20)(6,21)(7,23)(8,17)(9,18)(10,22)(11,19)(12,24)(13,14), (1,2)(3,22)(4,19,12,23)(5,21,6,20)(7,24,11,16)(8,17,9,18)(10,15)(13,14),
  (1,3,13,15)(2,10,14,22)(4,19,16,7)(5,8,17,20)(6,9,18,21)(11,12,23,24), (1,3,14,22)(2,10,13,15)(5,8,18,21)(6,9,17,20)(7,19)(11,23),
  (1,3,14,22)(2,10,13,15)(4,12)(5,9,18,20)(6,8,17,21)(7,23)(11,19)(16,24), (1,5,15,20,14,18,10,9)(2,6,22,21,13,17,3,8)(4,12,16,24)(7,19),
  (1,5,22,20,14,18,3,9)(2,6,15,21,13,17,10,8)(4,11,12,19)(7,16,23,24), (1,5,13,17)(2,6,14,18)(3,20,15,8)(4,11,16,23)(7,12,19,24)(9,10,21,22),
  (1,5,14,18)(2,6,13,17)(3,20,22,9)(4,19)(7,16)(8,10,21,15)(11,12)(23,24), (1,5,13,17)(2,6,14,18)(3,21,15,9)(4,12,16,24)(7,11,19,23)(8,10,20,22),
  (1,5,14,18)(2,6,13,17)(3,21,22,8)(4,16)(7,19)(9,10,20,15), (1,8,3,17,14,21,22,6)(2,9,10,18,13,20,15,5)(4,7,12,23)(11,16,19,24),
  (1,8,10,17,14,21,15,6)(2,9,3,18,13,20,22,5)(4,12,16,24)(11,23), (1,13)(2,14)(3,15)(4,16)(5,17)(6,18)(7,19)(8,20)(9,21)(10,22)(11,23)(12,24),
  (1,13)(2,14)(3,15)(4,24)(5,18)(6,17)(7,23)(8,21)(9,20)(10,22)(11,19)(12,16), (1,13)(2,14)(3,22)(4,19)(5,17)(6,18)(7,16)(8,21)(9,20)(10,15)(11,24)(12,23),
  (1,14)(2,13)(3,22)(5,18)(6,17)(8,21)(9,20)(10,15) ];
gg3s2!.Moonshinens:=[ 1, 2, 2, 4, 2, 2, 2, 4, 2, 4, 4, 8, 8, 2, 4, 2, 4, 8, 8, 1, 2, 2, 2 ];
gg3s2!.Moonshinehs:=[ 1, 1, 1, 1, 1, 2, 2, 2, 8, 1, 2, 1, 4, 8, 2, 8, 1, 4, 1, 4, 2, 2, 1 ];

# There are two cocycles that satisfy the conjugacy relations
gg3s2!.MoonshineVec:=[ 4, 2, 4, 6, 4, 2, 4, 6, 2, 4, 2, 2, 4, 0, 0, 4, 6, 4, 7, 1, 1, 2, 5, 6, 6, 2, 2, 4, 6, 2, 4, 0, 4, 2, 2 ];
gg3s2!.MoonshineVecPrime:=[ 4, 2, 4, 6, 4, 2, 4, 6, 2, 4, 2, 2, 4, 0, 0, 4, 6, 4, 3, 5, 5, 2, 1, 6, 6, 2, 2, 4, 6, 2, 4, 0, 4, 2, 2 ];

gg3s3!.MoonshineClassNames:=[ "1a", "3a", "3b", "3c", "3d", "3e", "3f", "3g", "3h", "3i", "3j" ];
gg3s3!.MoonshineClassReps:=[ (), (2,12,18)(3,23,16)(4,15,11)(5,8,19)(6,14,24)(7,17,20), (2,18,12)(3,16,23)(4,11,15)(5,19,8)(6,24,14)(7,20,17), (1,2,4)(3,22,6)(5,19,8)(7,20,17)(9,12,
    11)(10,18,15)(13,14,16)(21,24,23), (1,2,11)(3,21,24)(4,10,18)(6,16,22)(9,12,15)(13,14,23), (1,2,15)(3,13,14)(4,9,12)(5,8,19)(6,23,22)(7,17,20)(10,18,
    11)(16,21,24), (1,4,2)(3,6,22)(5,8,19)(7,17,20)(9,11,12)(10,15,18)(13,16,14)(21,23,24), (1,4,18)(2,9,11)(3,24,22)(5,19,8)(6,13,16)(7,20,17)(10,15,
    12)(14,21,23), (1,4,12)(2,10,15)(3,14,22)(6,21,23)(9,11,18)(13,16,24), (1,9,10)(2,12,18)(3,16,23)(4,11,15)(6,14,24)(13,21,22),
  (1,10,9)(2,18,12)(3,23,16)(4,15,11)(6,24,14)(13,22,21) ];
gg3s3!.Moonshinens:=[ 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 ];
gg3s3!.Moonshinehs:=[ 1, 1, 1, 3, 1, 3, 3, 3, 1, 1, 1 ];
gg3s3!.MoonshineVec:=[ 0, 2, 2, 0, 0, 1 ]; #unique
#ClassNames(CharacterTable(gg3s2));
#List(ConjugacyClasses(CharacterTable(gg3s3)),class->Representative(class));
#List(gg3s2!.MoonshineClassReps,class->gg3!.Moonshinehs[ConjugateToWhich(gg3,gg3!.MoonshineClassReps,class)]);
#List(gg3s2!.MoonshineClassReps,class->ConjugateToWhich(gg3,gg3!.MoonshineClassReps,class));


#------------------------------------------------#
# Construction of G^{(4)}
#------------------------------------------------#

gg4:=Group([ (2,3,4,5,6,7,8)(10,11,12,13,14,15,16), (1,7)(4,14)(5,13)(6,12)(8,16)(9,15) ]);
exp4:=12;
cc4:=[ 0, 0, 0, 0, 0, 0, 6, 18, 18, 6, 0, 6, 18, 6, 18, 6, 6, 18, 6, 6, 18, 6, 0, 0, 0, 0, 0, 0, 0,
  12, 0, 6, 3, 12, 6, 6, 12, 18, 6, 18, 6, 0, 9, 15, 12, 0, 12, 21, 12, 18, 12, 6, 0, 12, 0, 0, 0, 15,
  15, 3, 9, 12, 12, 3, 6, 12, 12, 21, 15, 0, 9, 9, 15, 18, 18, 3, 3, 3, 15, 12, 18, 21, 12, 12, 18, 6,
  3, 0, 15, 12, 15, 9, 15, 0, 6, 21, 3, 9, 0, 18, 12, 3, 18, 6, 6, 21, 12, 12, 12, 0, 0, 18, 18, 6, 0,
  12, 12, 0, 12, 12, 6, 12, 6, 6, 0, 6 ]; #or
cc4prime:=[ 0, 0, 0, 0, 0, 0, 6, 18, 18, 6, 0, 6, 18, 6, 18, 6, 6, 18, 6, 6, 18, 6, 0, 0, 0, 0, 0, 0, 0, 12, 0, 6, 15, 12, 6, 6, 12, 18, 6, 18, 6, 0, 21, 3, 12, 0, 12, 9, 12, 18, 12, 6, 0, 12, 0, 0, 0, 3, 3, 15, 21, 12, 12, 15, 6, 12, 12, 9, 3, 0, 21, 21, 3, 18, 18, 15, 15, 15, 3, 12, 18, 9, 12, 12, 18, 6, 15, 0, 3, 12, 3, 21, 3, 0, 6, 9, 15, 21, 0, 18, 12, 15, 18, 6, 6, 9, 12, 12, 12, 0, 0, 18, 18, 6, 0, 12, 12, 0, 12, 12, 6, 12, 6, 6, 0, 6 ];


#Resolution.  Takes ~10 minutes to calculate.
# N1:=gg4;
# N2:=Group([ (1,9)(2,10)(3,11)(4,12)(5,13)(6,14)(7,15)(8,16), (1,2)(3,7)(4,5)(6,8)(9,10)(11,15)(12,13)
# (14,16), (1,3)(2,7)(4,8)(5,6)(9,11)(10,15)(12,16)(13,14), (1,4)(2,5)(3,8)(6,7)(9,12)(10,13)(11,16)
# (14,15) ]); # Z_2^4
# N3:=Group(());
# if not IsBound(rr4) then rr4:=ResolutionNormalSeries([N1,N2,N3],4); fi;
# gg4!.MoonshineR:=rr4;

gg4!.Moonshinem:=4;
gg4!.MoonshineClassReps:=[ (), (1,9)(2,10)(3,11)(4,12)(5,13)(6,14)(7,15)(8,16), (1,15)(2,11)(3,10)(4,14)(5,16)(6,12)(7,9)(8,13),
(3,11)(4,13)(5,12)(6,8)(7,15)(14,16), (1,15,9,7)(2,11,10,3)(4,8,12,16)(5,6,13,14),
(1,16,2,14)(3,4,7,5)(6,9,8,10)(11,12,15,13), (3,4,6)(5,8,7)(11,12,14)(13,16,15),
(1,9)(2,10)(3,12,6,11,4,14)(5,16,7,13,8,15), (1,10)(2,9)(3,13,6,15,4,16)(5,14,7,12,8,11),
(1,2)(3,5,6,7,4,8)(9,10)(11,13,14,15,12,16), (1,9)(3,12,15,13)(4,7,5,11)(6,16,14,8),
(1,7,16,4,9,15,8,12)(2,11,14,13,10,3,6,5), (1,9)(2,11,4,13,6,15,8,10,3,12,5,14,7,16),
(2,3,4,5,6,7,8)(10,11,12,13,14,15,16), (1,9)(2,3,4,7,16,6,5,10,11,12,15,8,14,13),
(2,11,4,15,16,14,5)(3,12,7,8,6,13,10) ];

# I think that the choice of vector here is not unique, although I can't calculate the cocycles involving the elements of order 7 in
# a sensible amount of time, so it's possible that that would distinguish it.
# Given that the Sylow subgroup for 7 is Z/7Z, and the order seven element has n|h=7|1, it seems
# likely that the corresponding class is trivial.

# VecPrime known to be invalid because of restrictions on the spaces of theta functions corresponding (4B,4a).
# gg4!.MoonshineVecPrime:=[ 0, 0, 0, 0, 0, 0, 6, 18, 18, 6, 0, 6, 18, 6, 18, 6, 6, 18, 6, 6, 18, 6, 0, 0, 0, 0, 0, 0, 0, 12, 0, 6, 3, 12, 6,
#      6, 12, 18, 6, 18, 6, 0, 9, 15, 12, 0, 12, 21, 12, 18, 12, 6, 0, 12, 0, 0, 0, 15, 15, 3, 9, 12, 12, 3, 6, 12, 12, 21, 15, 0, 9, 9, 15, 18, 18, 3, 3, 3, 15, 12, 18, 21, 12, 12, 18, 6, 3, 0, 15, 12, 15,
#      9, 15, 0, 6, 21, 3, 9, 0, 18, 12, 3, 18, 6, 6, 21, 12, 12, 12, 0, 0, 18, 18, 6, 0, 12, 12, 0, 12, 12, 6, 12, 6, 6, 0, 6 ];
gg4!.MoonshineVec:=
 [ 0, 0, 0, 0, 0, 0, 6, 18, 18, 6, 0, 6, 18, 6, 18, 6, 6, 18, 6, 6, 18, 6, 0, 0, 0, 0, 0, 0, 0, 12, 0, 6, 15, 12, 6, 6, 12, 18, 6, 18, 6, 0, 21, 3, 12, 0, 12, 9, 12, 18, 12, 6, 0, 12, 0, 0, 0,
     3, 3, 15, 21, 12, 12, 15, 6, 12, 12, 9, 3, 0, 21, 21, 3, 18, 18, 15, 15, 15, 3, 12, 18, 9, 12, 12, 18, 6, 15, 0, 3, 12, 3, 21, 3, 0, 6,
     9, 15, 21, 0, 18, 12, 15, 18, 6, 6, 9, 12, 12, 12, 0, 0, 18, 18, 6, 0, 12, 12, 0, 12, 12, 6, 12, 6, 6, 0, 6 ];
gg4!.MoonshineClassNames :=[ "1a", "2a", "2b", "2c", "4a", "4b", "3a", "6a", "6b", "6c", "4c",
 "8a", "14a", "7a", "14b", "7b" ];
gg4!.Moonshinens := [1,1,2,2,2,4,3,3,6,6,4,4,7,7,7,7];
gg4!.Moonshinehs := [1,2,2,1,4,4,1,2,2,2,1,8,2,1,2,1];
gg4!.Moonshinev:=3; # mod 4

#------------------------------------------------#
# Construction of G^{(5)}
#------------------------------------------------#
gg5:=Group((1,2,3,4,5),(4,5)(6,7,8,9));
cc5:=[ 6, 3, 6, 0, 3, 6, 3, 3, 0, 6, 0, 9, 9, 0, 0, 9, 10, 4, 7, 9, 9 ];
if not IsBound(rr5) then rr5:=ResolutionFiniteGroup(gg5,4); fi;
exp5:=12;

gg5!.MoonshineR:=rr5;
gg5!.Moonshinem:=5;
gg5!.MoonshineClassReps:=[ (), (6,8)(7,9), (4,5)(6,7,8,9), (4,5)(6,9,8,7), (3,4,5), (3,4,5)(6,8)(7,9), (2,3)(4,5), (2,3)(4,5)(6,8)(7,9),
(2,3,4,5)(6,7,8,9), (2,3,4,5)(6,9,8,7), (1,2)(3,4,5)(6,7,8,9), (1,2)(3,4,5)(6,9,8,7), (1,2,3,4,5),
(1,2,3,4,5)(6,8)(7,9) ];
gg5!.MoonshineClassNames:=[ "1a", "2a", "4a", "4b", "3a", "6a", "2b", "2c", "4c", "4d", "12a", "12b", "5a", "10a" ];
gg5!.Moonshinens := [1,1,2,2,3,3, 2,2,4,4, 6, 6,5,5];
gg5!.Moonshinehs := [1,4,8,8,3,12,2,1,1,1,24,24,1,4];
gg5!.Moonshinev:=7; #mod 12
# gg5!.Moonshine24:=[24,0,0,0,0, 0,0,8,4,4, 0, 0,4,0];
gg5!.Moonshine24:=[24,24,0,0,0, 0,8,8,0,0, 0, 0,4,4];

#gg5 has a unique cocycle
gg5!.MoonshineVec:=[ 6, 3, 6, 0, 3, 6, 3, 3, 0, 6, 0, 9, 9, 0, 0, 9, 10, 4, 7, 9, 9 ];
gg5!.MoonshineExp:=12;

#------------------------------------------------#
# Construction of G^{(6)} = A5^4 D4
#------------------------------------------------#

#targetkappa := [0,20,12,0,6,18,18,12] # The 18 is a bit weird, but it shows in the numerical calculations
#gg6:=GeneralLinearGroup(2,3);
gg6:=Group([ (1,9)(2,3)(4,5)(6,38)(7,8)(10,11)(12,13)(14,46)(15,16)(17,33)(18,19)(20,21)(22,30)(23,24)(25,41)(26,27)(28,29)(31,32)(34,35)(36,37)(39,40)(42,43)(44,45)(47,48), (1,4,2)(3,43,19)(5,7,6)(8,24,48)(9,12,10)(11,35,27)(13,15,14)(16,32,40)(17,20,18)(21,23,22)(25,28,26)(29,31,30)(33,36,34)(37,39,38)(41,44,42)(45,47,46) ]);
if not IsBound(rr6) then rr6:=ResolutionFiniteGroup(gg6,4); fi;
# Trivial cocycle

gg6!.MoonshineR:=rr6;
gg6!.Moonshinem:=6;
gg6!.MoonshineClassReps:=[ (), (1,2,4)(3,19,43)(5,6,7)(8,48,24)(9,10,12)(11,27,35)(13,14,15)(16,40,32)(17,18,20)(21,22,23)(25,26,28)(29,30,31)(33,34,36)(37,38,39)(41,42,44)(45,46,47), (1,3,18,21,30,32,15,12)(2,5,38,40,31,28,41,43)(4,
    9,11,26,29,22,24,7)(6,8,47,44,25,27,34,37)(10,13,46,48,23,20,33,35)(14,16,39,36,17,19,42,45), (1,5,8,23,30,28,27,10)(2,9,13,16,31,22,20,19)(3,42,25,29,32,39,6,4)(7,
    38,36,35,26,41,45,48)(11,34,17,21,24,47,14,12)(15,46,44,43,18,33,37,40), (1,6)(2,10)(3,44)(4,19)(5,48)(7,39)(8,21)(9,14)(11,36)(12,27)(13,40)(15,47)(16,29)(17,22)(18,34)(20,43)(23,31)(24,45)(25,30)(26,
    42)(28,35)(32,37)(33,38)(41,46), (1,8,30,27)(2,13,31,20)(3,25,32,6)(4,42,29,39)(5,23,28,10)(7,36,26,45)(9,16,22,19)(11,17,24,14)(12,34,21,47)(15,44,18,37)(33,40,46,43)(35,41,48,38),
  (1,14,41,30,17,38)(2,44,24,31,37,11)(3,10,36,32,23,45)(4,48,15,29,35,18)(5,19,34,28,16,47)(6,33,22,25,46,9)(7,21,43,26,12,40)(8,39,13,27,42,20), (1,30)(2,31)(3,32)(4,29)(5,28)(6,25)(7,26)(8,27)(9,
    22)(10,23)(11,24)(12,21)(13,20)(14,17)(15,18)(16,19)(33,46)(34,47)(35,48)(36,45)(37,44)(38,41)(39,42)(40,43) ];
gg6!.MoonshineClassNames:=[ "1a", "3a", "8a", "8b", "2a", "4a", "6a", "2b" ];
gg6!.Moonshinens := [1,3,4,4,2,2,3,1];
gg6!.Moonshinehs := [1,1,2,2,1,2,2,2];
gg6!.Moonshinev:=0; #mod 1

gg6!.MoonshineVec:=[ 0, 0, 0, 0, 0, 0 ];
gg6!.MoonshineVecPrime:=[ 0, 0, 0, 12, 12, 0 ];
gg6!.MoonshineExp:=24;

#------------------------------------------------#
# Construction of G^{(6+3)}
#------------------------------------------------#

gg63 := Group([ (2,6)(3,5)(4,10)(8,14)(9,13)(11,17)(15,16), (1,2,7,11,4)(3,8,15,17,10)(5,9,16,18,12) ]);
gg63!.Moonshinev:= -1; #mod 6 #there's a weirdness in the multiplier system in that it is always e(vcd/nh) even if h does not divide n
gg63!.MoonshineClassNames:=[ "1a", "3a", "3b", "2a", "6a", "5a", "15a", "15b", "2b", "4a", "6b", "4b", "12a", "3c", "2c", "6c" ];
gg63!.MoonshineClassReps:=[ (), (1,5,3)(2,9,8)(4,12,10)(6,14,13)(7,16,15)(11,18,17), (2,8,9)(4,12,10)(6,7,11)(13,15,17)(14,16,18), (4,14)(6,10)(7,17)(11,16)(12,13)(15,18), (1,5,3)(2,9,8)(4,13,10,14,12,6)(7,11,15,17,16,18),
  (2,10,14,7,18)(4,13,16,17,9)(6,15,11,8,12), (1,5,3)(2,4,6,7,17,8,10,13,15,18,9,12,14,16,11), (1,3,5)(2,12,13,7,11,9,10,6,16,18,8,4,14,15,17), (2,8)(3,5)(4,10)(6,14)(7,18)(11,16)(15,17),
  (3,5)(4,13,7,11)(6,15,18,10)(8,9)(12,14,16,17), (2,4,9,10,8,12)(3,5)(6,16,11,14,7,18)(13,15,17), (1,2)(3,8)(4,14,7,17)(5,9)(6,15,18,10)(11,12,13,16), (1,9,3,2,5,8)(4,13,15,17,12,6,7,11,10,14,16,18),
  (1,2,12)(3,8,4)(5,9,10)(6,7,18)(11,13,15)(14,16,17), (1,2)(3,9)(4,13)(5,8)(6,10)(7,11)(12,14)(15,18)(16,17), (1,2,10,13,7,18)(3,9,12,6,15,17)(4,14,16,11,5,8) ];
gg63!.Moonshinens := [1,1,3,2,2,5,5,5,2,4,6,4,4,3,  2,6];
gg63!.Moonshinehs := [1,3,3,1,3,1,3,3,2,1,6,2,6,1,101,1];
gg63!.Moonshinem:=63;

N1:=gg63;
N2:=Group([ (1,3,5)(2,7,6)
  (8,15,13)(9,16,14)(11,18,17), (2,8,9)(4,12,10)(6,7,11)(13,15,17)(14,16,18), (1,3,5)(2,9,8)(4,14,17)
  (6,18,10)(11,12,13), (1,6,4)(2,9,8)(3,13,10)(5,14,12)(11,17,18), (1,5,3)(2,9,8)(4,12,10)(6,14,13)
  (7,16,15)(11,18,17) ]);
N3:=Group([ (1,5,3)(2,9,8)(4,12,10)(6,14,13)(7,16,15)(11,18,17) ]);
N4:=Group(());
if not IsBound(rr63) then rr63:=ResolutionNormalSeries([N1,N2,N3,N4],4); fi;
gg63!.MoonshineR:=rr63;

#this is unique, assuming I didn't screw something up.
#gg63!.MoonshineVec:=[ 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 10, 0, 0, 6, 8, 6, 2, 8, 0, 6, 4, 2, 0, 4, 8, 0, 6, 8, 10, 10, 10, 0, 8, 6, 6, 0, 8, 4, 0, 0, 4, 0, 0, 10, 10, 10, 4, 0, 6, 0, 6, 0 ];
gg63!.MoonshineVec:=[ 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 10, 0, 0, 6, 8, 6, 2, 8, 0, 6, 4, 2, 0, 4, 8, 0, 6, 8, 10, 10, 10, 0, 8, 6, 6, 0, 8, 4, 0, 0, 4, 0, 0, 10, 10, 10, 4, 0, 6, 0, 6, 0 ];
#------------------------------------------------#
# Construction of G^{(7)} = A6^4
#------------------------------------------------#

#targetkappas := [0,18,0,12,12,12,0];
gg7:=Group([ (1,17,9)(2,5,3)(4,20,12)(6,24,23)(7,14,8)(10,13,11)(15,22,16)(18,21,19), (1,7,4,2)(3,21,8,6)(5,16,14,11)(9,15,12,10)(13,24,22,19)(17,23,20,18) ]); #SpecialLinearGroup(2,3);
cc7:=[ 0, 18, 18, 3, 6, 15, 6 ]; # or
#cc7:=[ 0, 18, 18, 15, 6, 3, 6 ];
exp7:=24;
if not IsBound(rr7) then rr7:=ResolutionFiniteGroup(gg7,4); fi;


gg7!.MoonshineClassReps:=[ (), (1,2,4,7)(3,6,8,21)(5,11,14,16)(9,10,12,15)(13,19,22,24)(17,18,20,23), (1,3,23)(2,12,16)(4,8,18)(5,13,21)(6,
  14,22)(7,9,11)(10,20,24)(15,17,19), (1,4)(2,7)(3,8)(5,14)(6,21)(9,12)(10,15)(11,16)(13,22)(17,20)(18,23)(19,24)
  , (1,5,10,4,14,15)(2,20,6,7,17,21)(3,24,11,8,19,16)(9,13,18,12,22,23), (1,6,16,4,21,11)(2,15,18,7,10,23)(3,17,
    22,8,20,13)(5,19,9,14,24,12), (1,14,10)(2,17,6)(3,19,11)(4,5,15)(7,20,21)(8,24,16)(9,22,18)(12,13,23) ];
gg7!.MoonshineClassNames:=[ "1a", "4a", "3a", "2a", "6a", "6b", "3b" ];
gg7!.Moonshinens:=[1,2,3,1,3,3,3];
gg7!.Moonshinehs:=[1,8,1,4,4,4,1];
gg7!.MoonshineR := rr7;
gg7!.Moonshinev:=1; # mod 4
gg7!.Moonshinem:=7;

#both of these are consistant up to the e,g multiplier system.
gg7!.MoonshineVec := [ 0, 18, 18, 3, 6, 15, 6 ];
gg7!.MoonshineVecPrime := [ 0, 18, 18, 15, 6, 3, 6 ];

#------------------------------------------------#
# Construction of G^{(8)}
#------------------------------------------------#

gg8:=Group([ (1,2)(3,8)(4,6)(5,7), (1,3,4,7)(2,5,6,8), (1,4)(2,6)(3,7)(5,8) ]); #DihedralGroup(8);
if not IsBound(rr8) then rr8:=ResolutionFiniteGroup(gg8,4); fi;
cc8:=[0,2,0,0];

# NB: 2b is the central element, not 2a
gg8!.MoonshineClassReps:=[ (), (1,2)(3,8)(4,6)(5,7), (1,3,4,7)(2,5,6,8), (1,4)(2,6)(3,7)(5,8), (1,5)(2,3)(4,8)(6,7) ];
gg8!.MoonshineClassNames:=[ "1a", "2a", "4a", "2b", "2c" ];
gg8!.Moonshinens:=[1, 2, 2, 1, 2 ]; # 2a and 2c (2B and 2C resp. in UMatNL) have different chis, but the same n|h
gg8!.Moonshinehs:=[1, 1, 4, 2, 1 ];
gg8!.MoonshineR := rr8;
gg8!.Moonshinev:=-1; #mod 2
gg8!.MoonshineVec:=cc8; #[ 0, 2, 0, 0 ]
gg8!.Moonshinem:=8;

#------------------------------------------------#
# Construction of G^{(9)}
#------------------------------------------------#

#targetkappas :=   [0,,1,,,3];
gg9:=Group([ (1,2)(3,12)(4,10)(5,11)(6,8)(7,9), (1,3,4,7,8,11)(2,5,6,9,10,12), (1,4,8)(2,6,10)(3,7,11)(5,9,12) ]); #DihedralGroup(12);
cc9:=[ 0, 3, 0, 5 ]; #, where we have made the choice that 2B in the paper is 2a in GAP or
#cc9:=[ 3, 0, 3, 2 ]; # with the choice that 2B in the paper is 2b in GAP so that 2C becomes 2a.
exp9:=6;
if not IsBound(rr9) then rr9:=ResolutionFiniteGroup(gg9,4); fi;

gg9!.MoonshineClassReps:=[ (), (1,2)(3,12)(4,10)(5,11)(6,8)(7,9), (1,3,4,7,8,11)(2,5,6,9,10,12), (1,4,8)(2,6,10)(3,7,11)(5,9,12),
(1,5)(2,3)(4,12)(6,11)(7,10)(8,9), (1,7)(2,9)(3,8)(4,11)(5,10)(6,12) ];
gg9!.MoonshineClassNames:=[ "1a", "2a", "6a", "3a", "2b", "2c" ];
gg9!.Moonshinens := [1,2, 3,3,2,1];
gg9!.Moonshinehs := [1,2,12,3,1,4];
gg9!.MoonshineR := rr9;
gg9!.Moonshinev:=-1; # mod 6
gg9!.MoonshineVec:=[ 3, 0, 3, 2 ]; #unique
gg9!.MoonshineExp:=6;
gg9!.Moonshinem:=9;


#------------------------------------------------#
# Construction of G^{(10)}
#------------------------------------------------#

gg10:=Group((1,2,3,4));
if not IsBound(rr10) then rr10:=ResolutionFiniteGroup(gg10,4); fi;

gg10!.MoonshineClassReps:=[ (), (1,2,3,4), (1,3)(2,4), (1,4,3,2) ];
gg10!.MoonshineClassNames:=[ "1a", "4a", "2a", "4b" ];
gg10!.Moonshinens := [1,2,1,2];
gg10!.Moonshinehs := [1,2,2,2];
gg10!.MoonshineR := rr10;
gg10!.Moonshinev:=1; #mod 1
gg10!.MoonshineVec:=[ 0 ]; #unique
gg10!.MoonshineExp:=4;
gg10!.Moonshinem:=10;

#------------------------------------------------#
# Construction of G^{(10+5)} =D6^4
#------------------------------------------------#

gg105:=SymmetricGroup([1..4]);
if not IsBound(rr105) then rr105:=ResolutionFiniteGroup(gg105,4); fi;

gg105!.MoonshineClassReps:=[ (), (1,2), (1,2)(3,4),(1,2,3),(1,2,3,4) ];
gg105!.MoonshineClassNames:=[ "1a", "2a", "2b","3a", "4a" ];
gg105!.Moonshinens := [1,2,2,3,4];
gg105!.Moonshinehs := [1,1,2,1,4];
gg105!.MoonshineR := rr105;
gg105!.Moonshinev:=1; #mod 4
gg105!.MoonshineVec:=[ 0 ]; #unique
gg105!.MoonshineExp:=4;
gg105!.Moonshinem:=105;

#------------------------------------------------#
# Construction of G^{(12)}
#------------------------------------------------#

gg12:=Group((1,2));
if not IsBound(rr12) then rr12:=ResolutionFiniteGroup(gg12,4); fi;

gg12!.MoonshineClassReps:=[ (), (1,2) ];
gg12!.MoonshineClassNames:=[ "1a", "2a" ];
gg12!.Moonshinens := [1,1];
gg12!.Moonshinehs := [1,2];
gg12!.MoonshineR := rr12;
gg12!.Moonshinev:=0; #mod 1
gg12!.MoonshineVec:=[ 0 ]; #unique
gg12!.MoonshineExp:=2;
gg12!.Moonshinem:=12;

#------------------------------------------------#
# Construction of G^{(12+4)}
#------------------------------------------------#

gg124:=Group([ (1,9)(2,3)(4,5)(6,38)(7,8)(10,11)(12,13)(14,46)(15,16)(17,33)(18,19)(20,21)(22,30)(23,24)(25,41)(26,27)(28,29)(31,32)(34,35)(36,37)(39,40)(42,43)(44,45)(47,48), (1,4,2)(3,43,19)(5,7,6)(8,24,48)(9,12,10)(11,35,27)(13,15,14)(16,32,40)(17,20,18)(21,23,22)(25,28,26)(29,31,30)(33,36,34)(37,39,38)(41,44,42)(45,47,46) ]);


#targetkappas = [ 0, 0, 6, 6, 0, 12, 0, 0 ];
cc124:= [ 0, 0, 0, 18, 6, 0 ];
if not IsBound(rr124) then rr124:=ResolutionFiniteGroup(gg124,4); fi;

gg124!.MoonshineClassReps:=[ (), (1,2,4)(3,19,43)(5,6,7)(8,48,24)(9,10,12)(11,27,35)(13,14,15)(16,40,32)(17,18,20)(21,22,23)(25,26,28)(29,30,31)(33,34,36)(37,38,39)(41,42,44)(45,46,47), (1,3,18,21,30,32,15,12)(2,5,38,40,31,28,41,43)(4,
    9,11,26,29,22,24,7)(6,8,47,44,25,27,34,37)(10,13,46,48,23,20,33,35)(14,16,39,36,17,19,42,45), (1,5,8,23,30,28,27,10)(2,9,13,16,31,22,20,19)(3,42,25,29,32,39,6,4)(7,
    38,36,35,26,41,45,48)(11,34,17,21,24,47,14,12)(15,46,44,43,18,33,37,40), (1,6)(2,10)(3,44)(4,19)(5,48)(7,39)(8,21)(9,14)(11,36)(12,27)(13,40)(15,47)(16,29)(17,22)(18,34)(20,43)(23,31)(24,45)(25,30)(26,
    42)(28,35)(32,37)(33,38)(41,46), (1,8,30,27)(2,13,31,20)(3,25,32,6)(4,42,29,39)(5,23,28,10)(7,36,26,45)(9,16,22,19)(11,17,24,14)(12,34,21,47)(15,44,18,37)(33,40,46,43)(35,41,48,38),
  (1,14,41,30,17,38)(2,44,24,31,37,11)(3,10,36,32,23,45)(4,48,15,29,35,18)(5,19,34,28,16,47)(6,33,22,25,46,9)(7,21,43,26,12,40)(8,39,13,27,42,20), (1,30)(2,31)(3,32)(4,29)(5,28)(6,25)(7,26)(8,27)(9,
    22)(10,23)(11,24)(12,21)(13,20)(14,17)(15,18)(16,19)(33,46)(34,47)(35,48)(36,45)(37,44)(38,41)(39,42)(40,43) ];

# NB: 2b is the central element
gg124!.MoonshineClassNames:=[ "1a", "3a", "8a", "8b", "2a", "4a", "6a", "2b" ];
gg124!.Moonshinens := [1,3,4,4,2,2,3,1];
gg124!.Moonshinehs := [1,1,8,8,1,4,2,2];
gg124!.MoonshineR := rr124;
gg124!.Moonshinev:= -1; #mod 4
gg124!.MoonshineVec:=[ 0, 0, 0, 18, 6, 0 ]; #unique
gg124!.MoonshineExp:=24;
gg124!.Moonshinem:=124;

#------------------------------------------------#
# Construction of G^{(13)}
#------------------------------------------------#

gg13:=Group((1,2,3,4));
cc13:=[1];
exp13:=4;
if not IsBound(rr13) then rr13:=ResolutionFiniteGroup(gg13,4); fi;

gg13!.MoonshineR:=rr13;
gg13!.MoonshineClassReps:=[ (), (1,2,3,4), (1,3)(2,4), (1,4,3,2) ];
gg13!.MoonshineClassNames:=[ "1a", "4a", "2a", "4b" ];
gg13!.Moonshinens:=[1,2,1,2];
gg13!.Moonshinehs:=[1,8,4,8];
gg13!.Moonshinev:=7;  #should be -1 mod 4  why is this 7?
gg13!.MoonshineVec:=[1]; #unique
gg13!.MoonshineExp:=4;
gg13!.Moonshinem:=13;

#------------------------------------------------#
# Construction of G^{(14+7)}
#------------------------------------------------#

gg147:=SymmetricGroup([1,2,3]);
if not IsBound(rr147) then rr147:=ResolutionFiniteGroup(gg147,4); fi;

gg147!.MoonshineR:=rr147;
gg147!.MoonshineClassReps:=[ (), (1,2), (1,2,3) ];
gg147!.MoonshineClassNames:=[ "1a", "2a", "3a" ];
gg147!.Moonshinens:=[1,2,3];
gg147!.Moonshinehs:=[1,1,3];
gg147!.Moonshinev:=1; #mod 3
gg147!.MoonshineVec:=[ 0, 0, 2, 0 ]; #unique
gg147!.MoonshineExp:=6;
gg147!.Moonshinem:=147;

#------------------------------------------------#
# Construction of G^{(16)}
#------------------------------------------------#

gg16:=Group((1,2));
if not IsBound(rr16) then rr16:=ResolutionFiniteGroup(gg16,4); fi;

gg16!.MoonshineR:=rr16;
gg16!.MoonshineClassReps:=[ (), (1,2)];
gg16!.MoonshineClassNames:=[ "1a", "2a"];
gg16!.Moonshinens:=[1,1];
gg16!.Moonshinehs:=[1,2];
gg16!.Moonshinev:=0; #mod 1
gg16!.MoonshineVec:=[0]; #unique
gg16!.MoonshineExp:=2;
gg16!.Moonshinem:=16;

#------------------------------------------------#
# Construction of G^{(18)}
#------------------------------------------------#

gg18:=Group((1,2));
if not IsBound(rr18) then rr18:=ResolutionFiniteGroup(gg18,4); fi;

gg18!.MoonshineR:=rr18;
gg18!.MoonshineClassReps:=[ (), (1,2)];
gg18!.MoonshineClassNames:=[ "1a", "2a"];
gg18!.Moonshinens:=[1,1];
gg18!.Moonshinehs:=[1,2];
gg18!.Moonshinev:=0; #mod 1
gg18!.MoonshineVec:=[0]; #unique
gg18!.MoonshineExp:=2;
gg18!.Moonshinem:=18;

#------------------------------------------------#
# Construction of G^{(18+9)}
#------------------------------------------------#

gg189:=Group((1,2));
if not IsBound(rr189) then rr189:=ResolutionFiniteGroup(gg189,4); fi;

gg189!.MoonshineR:=rr189;
gg189!.MoonshineClassReps:=[ (), (1,2) ];
gg189!.MoonshineClassNames:=[ "1a", "2a"];
gg189!.Moonshinens:=[1,2];
gg189!.Moonshinehs:=[1,1];
gg189!.Moonshinev:=0; #mod 1        #1; #mod 2.
gg189!.MoonshineVec:=[1]; #unique
gg189!.MoonshineExp:=2;
gg189!.Moonshinem:=189;

#------------------------------------------------#
# Construction of G^{(22+11)}
#------------------------------------------------#

gg2211:=Group((1,2));
if not IsBound(rr2211) then rr2211:=ResolutionFiniteGroup(gg2211,4); fi;
gg2211!.MoonshineR:=rr2211;
gg2211!.MoonshineClassReps:=[ (), (1,2) ];
gg2211!.MoonshineClassNames:=[ "1a", "2a"];
gg2211!.Moonshinens:=[1,2];
gg2211!.Moonshinehs:=[1,2];
gg2211!.Moonshinev:=1; #mod 2.
gg2211!.MoonshineVec:=[1]; #unique
gg2211!.MoonshineExp:=2;
gg2211!.Moonshinem:=2211;


#------------------------------------------------#
# Construction of G^{(25)} = A24
#------------------------------------------------#

gg25:=Group((1,2));
cc25:=[1];
if not IsBound(rr25) then rr25:=ResolutionFiniteGroup(gg25,4); fi;
gg25!.MoonshineR:=rr25;
gg25!.MoonshineClassReps:=[ (), (1,2) ];
gg25!.MoonshineClassNames:=[ "1a", "2a"];
gg25!.Moonshinens:=[1,1];
gg25!.Moonshinehs:=[1,4];
gg25!.Moonshinev:=1; #mod 2.
gg25!.MoonshineVec:=[1]; #unique
gg25!.MoonshineExp:=2;
gg25!.Moonshinem:=25;

#------------------------------------------------#
# Construction of G^{(30+15)}
#------------------------------------------------#

gg3015:=Group(());
if not IsBound(rr3015) then rr3015:=ResolutionFiniteGroup(gg3015,4); fi;
gg3015!.MoonshineR:=rr3015;
gg3015!.MoonshineClassReps:=[ () ];
gg3015!.MoonshineClassNames:=[ "1a"];
gg3015!.Moonshinens:=[1];
gg3015!.Moonshinehs:=[1];
gg3015!.Moonshinev:=0; #mod 1.
gg3015!.Moonshinem:=3015;

#------------------------------------------------#
# Construction of G^{(30+6,10,15)}
#------------------------------------------------#

gg3061015:=SymmetricGroup([1,2,3]);
if not IsBound(rr3061015) then rr3061015:=ResolutionFiniteGroup(gg3061015,4); fi;

gg3061015!.MoonshineR:=rr3061015;
gg3061015!.MoonshineClassReps:=[ (), (1,2), (1,2,3) ];
gg3061015!.MoonshineClassNames:=[ "1a", "2a", "3a" ];
gg3061015!.Moonshinens:=[1,2,3];
gg3061015!.Moonshinehs:=[1,1,3];
gg3061015!.Moonshinev:= -1; #mod 3
gg3061015!.MoonshineVec:=[ 0, 0, 4, 0 ]; #unique
gg3061015!.MoonshineExp:=6;
gg3061015!.Moonshinem:=3061015;

#------------------------------------------------#
# Construction of G^{(46+23)}
#------------------------------------------------#

gg4623:=Group(());
if not IsBound(rr4623) then rr4623:=ResolutionFiniteGroup(gg4623,4); fi;
gg4623!.MoonshineR:=rr4623;
gg4623!.MoonshineClassReps:=[ () ];
gg4623!.MoonshineClassNames:=[ "1a"];
gg4623!.Moonshinens:=[1];
gg4623!.Moonshinehs:=[1];
gg4623!.Moonshinev:=0; #mod 1.
gg4623!.Moonshinem:=4623;
